use bevy_eventwork::{tcp::TcpProvider, ConnectionId, Network};
use std::{
	io::{stdin, stdout, Stdout},
	process,
	sync::{mpsc, Arc, Mutex},
	thread::{self, JoinHandle},
};
use termion::{
	cursor::{self, HideCursor},
	event::Key,
	input::TermRead,
	raw::{IntoRawMode, RawTerminal},
};

use bevy::prelude::*;

use crate::common::client_msg::Move;

use super::net::Server;

pub struct InputPlugin;

impl Plugin for InputPlugin {
	fn build(&self, app: &mut App) {
		let (rec, _thread) = spawn_reader();
		let rec = Arc::new(Mutex::new(rec));
		app.insert_resource(InputReceiver(rec))
			.add_event::<KeyEvent>()
			.add_systems(Update, try_read_keys)
			.add_systems(Update, on_move);
	}
}

#[derive(Resource)]
pub struct InputReceiver(Arc<Mutex<mpsc::Receiver<Key>>>);

#[derive(Debug, Event)]
pub struct KeyEvent(Key);

fn spawn_reader() -> (mpsc::Receiver<Key>, JoinHandle<()>) {
	let (tx, rx) = mpsc::channel();
	let thread = thread::spawn(move || {
		let stdout = stdout().into_raw_mode().unwrap();
		println!("{}", cursor::Hide);
		let mut keys = stdin().keys();
		while let Some(Ok(key)) = keys.next() {
			if key == Key::Esc {
				println!("exitting.");
				println!("{}", cursor::Restore);
				drop(stdout);
				process::exit(0);
			}
			tx.send(key).ok();
		}
	});
	(rx, thread)
}

fn try_read_keys(receiver: Res<InputReceiver>, mut writer: EventWriter<KeyEvent>) {
	let receiver = receiver.0.lock().expect("Poisoned threads doomed the state.");
	writer.send_batch(receiver.try_iter().map(KeyEvent));
}

fn on_move(mut keys: EventReader<KeyEvent>, server: Res<Server>, net: Res<Network<TcpProvider>>) {
	if let Some(server) = server.0 {
		for key in keys.read() {
			let dir = match key.0 {
				Key::Char('z') => IVec2::Y,
				Key::Char('s') => -IVec2::Y,
				Key::Char('q') => -IVec2::X,
				Key::Char('d') => IVec2::X,
				_ => continue,
			};
			net.send_message(server, Move(dir)).unwrap();
		}
	}
}
