use crate::common::{self, server_msg::MapUpdates};
use bevy::{prelude::*, utils::hashbrown::HashSet};
use bevy_eventwork::{tcp::TcpProvider, ConnectionId, Network};

use super::{
	physics::{update_physics_collisions, Pos},
	player::Player,
};

pub struct DisplayPlugin;

impl Plugin for DisplayPlugin {
	fn build(&self, app: &mut App) {
		app.add_event::<RenderUpdate>()
			// .add_systems(Update, rerender_players)
			.add_systems(Update, on_updates.after(update_physics_collisions));
	}
}

#[derive(Debug, Component)]
pub struct Display(pub (char, char));

#[derive(Debug, Event)]
pub struct RenderUpdate(pub IVec2);

const RENDER_DISTANCE: i32 = common::RENDER_DISTANCE;
const RENDER_DISTANCE_SQ: i32 = RENDER_DISTANCE * RENDER_DISTANCE;

fn on_updates(
	mut updates: EventReader<RenderUpdate>,
	players: Query<(&Player, &Pos)>,
	displayables: Query<(&Pos, &Display)>,
	net: Res<Network<TcpProvider>>,
) {
	let mut to_render = HashSet::new();
	for RenderUpdate(event_pos) in updates.read() {
		for (Player(connection_id), player_pos) in &players {
			if player_pos.pos().distance_squared(*event_pos) <= RENDER_DISTANCE_SQ {
				to_render.insert((*connection_id, player_pos.pos()));
			}
		}
	}

	for (id, pos) in to_render {
		render(&net, id, pos, &displayables);
	}
}

fn render(
	net: &Res<Network<TcpProvider>>,
	id: ConnectionId,
	player_pos: IVec2,
	displayables: &Query<(&Pos, &Display)>,
) {
	let updates = displayables
		.iter()
		.filter(|(pos, _)| pos.pos().distance_squared(player_pos) <= RENDER_DISTANCE_SQ)
		.map(|(pos, Display(display))| (pos.pos() - player_pos, *display))
		.collect();
	net.send_message(id, MapUpdates(updates)).ok();
}

fn rerender_players(players: Query<&Pos, With<Player>>, mut writer: EventWriter<RenderUpdate>) {
	for Pos(pos) in &players {
		writer.send(RenderUpdate(pos.as_ivec2()));
	}
}
