use bevy::prelude::*;

use super::{display::RenderUpdate, player};

pub struct PhysicsPlugin;

impl Plugin for PhysicsPlugin {
	fn build(&self, app: &mut App) {
		app.add_systems(
			Update,
			(
				update_physics_forces.after(player::handle_move),
				update_physics_center.after(update_physics_forces),
				update_physics_collisions.after(update_physics_center),
			),
		);
	}
}

#[derive(Debug, Component, Clone)]
pub struct Pos(pub Vec2);

impl Pos {
	pub fn fpos(&self) -> Vec2 {
		self.0
	}

	pub fn pos_center(&self) -> Vec2 {
		self.pos().as_vec2()
	}

	pub fn pos(&self) -> IVec2 {
		self.0.round().as_ivec2()
	}

	pub fn dir_to_center(&self) -> Vec2 {
		self.pos_center() - self.fpos()
	}
}

#[derive(Debug, Component)]
pub struct Mass(pub usize);

#[derive(Debug, Component)]
pub struct Forces(Vec2);

impl Forces {
	pub fn zero() -> Self {
		Self::new(Vec2::ZERO)
	}
	pub fn new(forces: Vec2) -> Self {
		Self(forces)
	}
	pub fn add(&mut self, force: Vec2) {
		self.0 += force;
	}
}

const MAX_MOVEMENT_PER_TICK: f32 = 0.3;

fn update_physics_forces(mut blocks: Query<(&mut Pos, &mut Forces)>, mut updater: EventWriter<RenderUpdate>) {
	for (mut pos, mut forces) in &mut blocks {
		let old_pos = pos.clone();

		let applied = forces.0;
		let applied = applied.clamp_length_max(MAX_MOVEMENT_PER_TICK);
		pos.0 += applied;
		forces.0 -= applied;

		if old_pos.pos() != pos.pos() {
			updater.send(RenderUpdate(old_pos.pos()));
		}
	}
}

pub fn update_physics_collisions(mut blocks: Query<(&mut Pos, &Mass)>) {
	let mut combinations = blocks.iter_combinations_mut();
	while let Some([(mut a_pos, Mass(a_mass)), (mut b_pos, Mass(b_mass))]) = combinations.fetch_next() {
		if a_pos.pos() == b_pos.pos() {
			let a_to_b = (b_pos.fpos() - a_pos.fpos()).normalize_or(Vec2::Y);
			let total_mass = (a_mass + b_mass) as f32;
			let a_mass_frac = (*a_mass) as f32 / total_mass;
			let b_mass_frac = (*b_mass) as f32 / total_mass;
			let a_movement = (-a_to_b) * b_mass_frac;
			let b_movement = (a_to_b) * a_mass_frac;
			a_pos.0 += a_movement;
			b_pos.0 += b_movement;
		}
	}
}

pub fn update_physics_center(mut blocks: Query<(&Pos, &mut Forces)>) {
	for (pos, mut forces) in &mut blocks {
		let dir = pos.pos_center() - pos.0;
		forces.add(dir * 0.75);
	}
}
