use crate::common::client_msg::Move;
use bevy::prelude::*;
use bevy_eventwork::{tcp::TcpProvider, AppNetworkMessage, ConnectionId, NetworkData};

use super::{
	display::{Display, RenderUpdate},
	physics::{Forces, Mass, Pos},
};

pub struct PlayerPlugin;

impl Plugin for PlayerPlugin {
	fn build(&self, app: &mut App) {
		app.listen_for_message::<Move, TcpProvider>()
			.add_event::<RenderUpdate>()
			.add_systems(Update, handle_move);
	}
}

#[derive(Debug, Component)]
pub struct Player(pub ConnectionId);

#[derive(Debug, Component)]
pub struct Dir(pub IVec2);

fn player_display_for_dir(dir: IVec2) -> (char, char) {
	match dir {
		IVec2::Y => ('&', '/'),
		IVec2::NEG_X => ('_', '&'),
		IVec2::X => ('&', '_'),
		_ => ('&', '\\'),
	}
}

pub fn spawn_player(cmd: &mut Commands, conn_id: ConnectionId, pos: IVec2) {
	cmd.spawn((
		Player(conn_id),
		Pos(pos.as_vec2()),
		Display(player_display_for_dir(IVec2::X)),
		Dir(IVec2::X),
		Forces::zero(),
		Mass(10),
	));
}

pub fn remove_player(cmd: &mut Commands, conn_id: ConnectionId, players: &Query<(Entity, &Player)>) {
	for (entity, player) in players {
		if player.0 == conn_id {
			cmd.entity(entity).despawn();
		}
	}
}

pub fn handle_move(
	mut messages: EventReader<NetworkData<Move>>,
	mut players: Query<(&Player, &mut Forces, &Pos, &mut Dir, &mut Display)>,
) {
	for message in messages.read() {
		let conn_id = message.source();
		for (Player(id), mut forces, pos, mut dir, mut display) in &mut players {
			if conn_id == id {
				let dir_movement = message.0.as_vec2().normalize_or_zero();
				let added_force = dir_movement + pos.dir_to_center();
				forces.add(added_force);

				let idir = dir_movement.as_ivec2();
				if !(idir == IVec2::ZERO) {
					dir.0 = idir;
					display.0 = player_display_for_dir(idir);
				}
			}
		}
	}
}
